Module: Kettle

Defined in:
lib/kettle/emoji_regex.rb,
lib/kettle/dev.rb,
lib/kettle/dev/version.rb,
lib/kettle/dev/ci_helpers.rb,
lib/kettle/dev/template_helpers.rb

Overview

Stolen lovingly from @ticky:

https://github.com/ticky/ruby-emoji-regex/blob/develop/lib/emoji_regex.rb
because that project is Ruby >= 2.4, and this project is Ruby >= 2.3.

That project is under the MIT license, same as this project:

https://github.com/ticky/ruby-emoji-regex/blob/develop/LICENSE.md

If the minimum_ruby_version numbers align, will switch to a real dependency. Wrapping it in a namespace to avoid collisions with the real emoji_regex.

Defined Under Namespace

Modules: Dev, EmojiRegex